"use strict";
/**
 * @module common
 */
/** End Typedoc Module Declaration */
const _ = require('lodash');
const metadata_1 = require('../metadata/metadata');
class RegistryEntity {
    constructor() {
    }
    /**
     * Get the metadata for the model (static side)
     * @returns {ModelMetadata}
     */
    static getMetadata() {
        const metadata = this.__metadata;
        if (metadata || !this.__metadataDefault) {
            return metadata;
        }
        return this.__metadataDefault;
    }
    /**
     * Get the metadata for the model (instance side)
     * Note this is the same as Model.getMetadata(), but more convenient if you don't know or have
     * access to the model name
     * @returns {ModelMetadata}
     */
    getMetadata() {
        return this.constructor.getMetadata();
    }
}
exports.RegistryEntity = RegistryEntity;
/**
 * The EntityRegistry is the core register for all entities that are annotated with the entity
 * decorators:
 * * [[Model|@Model]]
 * * [[Controller|@Controller]]
 * * [[Seeder|@Seeder]]
 * * [[Migration|@Migration]]
 * * [[Store|@Store]]
 * * [[Service|@Service]]
 *
 * Both the static classes and any associated metatadata are assigned to the registry by the
 * decorators
 */
class EntityRegistry {
    constructor() {
        /** The internal registry */
        this.registry = new Map();
    }
    static register(type, entity, metadata) {
        return this.root.register(type, entity, metadata);
    }
    static clearAll() {
        return this.root.clearAll();
    }
    /**
     * Associate an entity class with a type and any metadata if present
     * @param type
     * @param entity
     * @param metadata
     * @returns {EntityRegistry}
     */
    register(type, entity, metadata) {
        if (!this.registry.get(type)) {
            this.registry.set(type, new Map());
        }
        let typeRegistry = this.registry.get(type);
        if (metadata) {
            metadata_1.initializeMetadata(entity);
            _.merge(entity.__metadata, metadata);
        }
        typeRegistry.set(entity.name, entity);
        return this;
    }
    /**
     * Retrieve all entities associated with the given type
     * @param type
     * @returns {any}
     */
    getAllOfType(type) {
        if (!this.registry.has(type)) {
            return new Map();
        }
        return this.registry.get(type);
    }
    /**
     * Remove all of a given type
     * @param type
     * @returns {EntityRegistry}
     */
    clearType(type) {
        this.registry.get(type)
            .clear();
        return this;
    }
    /**
     * Clear the registry
     * @returns {EntityRegistry}
     */
    clearAll() {
        this.registry.clear();
        return this;
    }
    /**
     * Remove a specific entity by type
     * @param type
     * @param name
     * @returns {boolean}
     */
    removeByType(type, name) {
        return this.getAllOfType(type)
            .delete(name);
    }
    /**
     * Check if an entity is already present in the registry
     * @param type
     * @param name
     * @returns {boolean}
     */
    hasEntity(type, name) {
        return !!this.findByType(type, name);
    }
    /**
     * Find an entity given a known type and class name
     * @param type
     * @param name
     * @returns {RegistryEntityStatic}
     */
    findByType(type, name) {
        return this.getAllOfType(type)
            .get(name);
    }
    /**
     * Find all entities with a given class name
     * @param name
     * @returns {any}
     */
    findAllWithName(name) {
        let found = new Map();
        this.registry.forEach((entitySet, key) => {
            let search = entitySet.get(name);
            if (search) {
                found.set(key, search);
            }
        });
        if (!found.size) {
            return null;
        }
        return found;
    }
}
// Global root registry
EntityRegistry.root = new EntityRegistry();
exports.EntityRegistry = EntityRegistry;
//# sourceMappingURL=entityRegistry.js.map