"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
/**
 * @module common
 */
/** End Typedoc Module Declaration */
const core_1 = require('@angular/core');
const service_1 = require('./service');
(function (LogVerbosity) {
    LogVerbosity[LogVerbosity["none"] = 0] = "none";
    LogVerbosity[LogVerbosity["error"] = 1] = "error";
    LogVerbosity[LogVerbosity["info"] = 2] = "info";
    LogVerbosity[LogVerbosity["verbose"] = 3] = "verbose";
    LogVerbosity[LogVerbosity["silly"] = 4] = "silly";
})(exports.LogVerbosity || (exports.LogVerbosity = {}));
var LogVerbosity = exports.LogVerbosity;
/**
 * Abstract Logger class that should be used as the DI token for any logging. The implementation
 * should be defined by the provider definition
 *
 * Example
 *
 * main.ts
 * ```typescript
 *   let providers: ProviderDefinition[] = [
 *     {provide: Logger, useClass: ConsoleLogger},
 *   ];
 * ```
 * someService.service.ts
 * ```typescript
 *   @Injectable()
 *   @Service()
 *   export class SomeService extends AbstractService {
 *     constructor(protected logger: Logger) {
 *       logger.source('Some Service').debug('constructor initialized')
 *     }
 *   }
 * ```
 *
 * Logger provides methods for all log types defined by
 * [The Syslog Protocol (RFC 5424)](http://tools.ietf.org/html/rfc5424)
 *
 */
let Logger = class Logger extends service_1.AbstractService {
    constructor(impl) {
        super();
        this.impl = impl;
    }
    /**
     * Log emergency message(s) to the log output
     *
     * Example:
     * ```typescript
     * try {
     *   // something that might throw Error
     * catch(e){
     *   logger.emergency(e.message).debug(e.stack);
     * }
     * ```
     * @param args
     * @returns {Logger}
     */
    emergency(...args) {
        return this.log('emergency', LogVerbosity.error, ...args);
    }
    /**
     * Log alert message(s) to the log output
     * @param args
     * @returns {any}
     */
    alert(...args) {
        return this.log('alert', LogVerbosity.error, ...args);
    }
    /**
     * Log critical message(s) to the log output
     * @param args
     * @returns {any}
     */
    critical(...args) {
        return this.log('critical', LogVerbosity.error, ...args);
    }
    /**
     * Log error message(s) to the log output
     * @param args
     * @returns {any}
     */
    error(...args) {
        return this.log('error', LogVerbosity.error, ...args);
    }
    /**
     * Log warning message(s) to the log output
     * @param args
     * @returns {any}
     */
    warning(...args) {
        return this.log('warning', LogVerbosity.info, ...args);
    }
    /**
     * Log notice message(s) to the log output
     * @param args
     * @returns {any}
     */
    notice(...args) {
        return this.log('notice', LogVerbosity.info, ...args);
    }
    /**
     * Log info message(s) to the log output
     * @param args
     * @returns {any}
     */
    info(...args) {
        return this.log('info', LogVerbosity.info, ...args);
    }
    /**
     * Log debug message(s) to the log output
     * @param args
     * @returns {Logger}
     */
    debug(...args) {
        return this.log('debug', LogVerbosity.verbose, ...args);
    }
    /**
     * Given the log level and list of messages, and if the global verbosity level is satisified,
     * invoke the persist log method.
     * @param logLevel
     * @param verbosity
     * @param messages
     * @returns {any}
     */
    log(logLevel, verbosity, ...messages) {
        this.setVerbosity(verbosity);
        const globalVerbosity = LogVerbosity[process.env.LOG_LEVEL];
        if (typeof globalVerbosity == 'number' && globalVerbosity < this.verbosityLevel) {
            this.setVerbosity(undefined, true);
            return;
        }
        return this.persistLog(logLevel, messages)
            .setVerbosity(undefined, true); //reset the verbosity to undefined for the next log
    }
    /**
     * Set a string to denote the source of the log. The effect this has on the log output is up
     * to the implementing class.
     * @param sourceName
     * @returns {Logger}
     */
    setSource(sourceName) {
        this.sourceName = sourceName;
        return this;
    }
    /**
     * Sets the current log verbosity level, used to compare against the global value to determine
     * whether or not to persist the log. If a verbosity is already set, don't force unless requested.
     * to, as
     * @param level
     * @param force
     * @returns {Logger}
     */
    setVerbosity(level, force = false) {
        if (!force && !!this.verbosityLevel) {
            return this;
        }
        this.verbosityLevel = level;
        return this;
    }
    /**
     * Chainable method to allow forcing the log level to only show when log verbosity level is 'silly'
     *
     * Example:
     * ```typescript
     * try {
     *   //do something that throws error
     * } catch (e) {
     *   this.logger.error(e.message).silly.debug(e.stack);
     * }
     * ```
     * This will only log the stack trace if the global `process.env.LOG_LEVEL` is `'silly'`
     * @returns {Logger}
     */
    get silly() {
        return this.setVerbosity(LogVerbosity.silly, true);
    }
    /**
     * Chainable method to allow forcing the log level to only show when log verbosity level is 'verbose'
     * @returns {Logger}
     */
    get verbose() {
        return this.setVerbosity(LogVerbosity.verbose, true);
    }
    /**
     * Creates a new instance of the logger class with the source set. This allows the use of
     * passing around log instances to avoid having to set source each time.
     *
     * Example:
     *
     * example.service.ts
     * ```typescript
     * export class ExampleService extends AbstractService {
     *   protected logger: Logger;
     *   constructor(loggerBase:Logger){
     *     this.logger = loggerBase.source(this.constructor.name);
     *   }
     *
     *   public logExample():void {
     *     this.logger.info('Log message');
     *   }
     * ```
     *
     * When the `(new ExampleService).logExample()` method is called, the logger implementation will
     * have the source defined as `ExampleService`.
     * @param source
     * @returns {any}
     */
    source(source) {
        return new this.impl().setSource(source);
    }
};
Logger = __decorate([
    core_1.Injectable(), 
    __metadata('design:paramtypes', [Object])
], Logger);
exports.Logger = Logger;
//# sourceMappingURL=logger.service.js.map