"use strict";
/**
 * @module server
 */
/** End Typedoc Module Declaration */
require('reflect-metadata');
const core_1 = require('@angular/core');
const abstract_server_1 = require('../servers/abstract.server');
const logger_service_1 = require('../../common/services/logger.service');
const main_1 = require('../main');
const controllers_bootstrapper_1 = require('./controllers.bootstrapper');
const models_bootstrapper_1 = require('./models.bootstrapper');
const seeders_bootstrapper_1 = require('./seeders.bootstrapper');
const migrations_bootstrapper_1 = require('./migrations.bootstrapper');
const services_bootstrapper_1 = require('./services.bootstrapper');
const logger_service_mock_1 = require('../../common/services/logger.service.mock');
let deferredLogs = [];
function deferredLog(level, ...messages) {
    deferredLogs.push({ level, messages });
}
exports.deferredLog = deferredLog;
/**
 * Handles the logging of any error encountered during bootstrap. If there is an instance of [[Logger]]
 * available, it is used, otherwise a fallback to console.error is used.
 *
 * In either case, all of the deferred logs are output first, then the error is logged and the
 * process is aborted
 * @param e
 * @param logger
 */
function handleBootstrapError(e, logger) {
    if (logger) {
        deferredLogs.forEach((log) => {
            logger[log.level](...log.messages);
        });
        logger.critical(e.constructor.name, e.message)
            .debug(e.stack);
        if (logger instanceof logger_service_mock_1.LoggerMock) {
            console.log('Logger is mock but a critical error occurred, outputting to console');
            console.error(e);
        }
    }
    else {
        console.error('Failed to initialize Logger, falling back to console');
        deferredLogs.forEach((log) => {
            console.log(log.level, ...log.messages);
        });
        console.error(e);
    }
    process.exit(1);
}
/**
 * The core bootstrapping factory function.
 *
 * This function resolves all of the providers, invokes all boostrappers, and finally returns the
 * [[Server]] instance for the bootstrapping start file to call [[Server.start]] on to start the
 * server running
 * @see http://zeroth.io/guide/application-lifecycle/#startup for detailed documentation
 * @param loadClasses
 * @param providers
 * @param afterBootstrap
 * @returns {function(): Promise<BootstrapResponse>}
 */
function bootstrap(loadClasses = [], providers = [], afterBootstrap) {
    let logger;
    deferredLog('debug', 'Classes loaded from app', loadClasses.reduce((all, classDict) => {
        return all.concat(Object.keys(classDict));
    }, []));
    return () => {
        deferredLog('info', 'Bootstrapping server');
        return Promise.all(main_1.CORE_PROVIDERS.concat(providers))
            .then((providers) => {
            //initialize all bootstrappers (in order they need to be created)
            const resolvedBootstrappers = [
                new models_bootstrapper_1.ModelBootstrapper,
                new services_bootstrapper_1.ServiceBootstrapper,
                new migrations_bootstrapper_1.MigrationBootstrapper,
                new seeders_bootstrapper_1.SeederBootstrapper,
                new controllers_bootstrapper_1.ControllerBootstrapper,
            ];
            const bootrapperProviders = resolvedBootstrappers.reduce((result, bootstrapper) => {
                return result.concat(bootstrapper.getInjectableEntities());
            }, []);
            const mergedProviders = core_1.ReflectiveInjector.resolve(bootrapperProviders.concat(providers));
            const injector = core_1.ReflectiveInjector.fromResolvedProviders(mergedProviders);
            logger = injector.get(logger_service_1.Logger)
                .source('bootstrap');
            // iterate over any logs that have been deferred
            deferredLogs.forEach((log) => {
                logger[log.level](...log.messages);
            });
            deferredLogs = []; //clear buffer
            // iterate over all of the resolved bootstrappers, invoking the bootstrap to initialize the
            // entities. iteration is completed in serial so that bootstrappers that depend on other
            // entities won't invoke until those other entities are finished intializing
            return resolvedBootstrappers.reduce((current, next) => {
                return current.then(() => {
                    return Promise.resolve(next.setInjector(injector)
                        .invokeBootstrap());
                });
            }, Promise.resolve()) //initial value
                .then(() => {
                // get vars for the bootstrapper
                const server = injector.get(abstract_server_1.Server);
                let response = { injector, server, logger };
                if (afterBootstrap) {
                    return Promise.resolve(afterBootstrap(response))
                        .then(() => response);
                }
                return response;
            });
        })
            .catch((e) => {
            handleBootstrapError(e, logger);
        });
    };
}
exports.bootstrap = bootstrap;
//# sourceMappingURL=bootstrap.js.map