"use strict";
const remoteCli_service_mock_1 = require('../services/remoteCli.service.mock');
const remoteCli_service_1 = require('../services/remoteCli.service');
const abstract_server_mock_1 = require('../servers/abstract.server.mock');
const logger_service_1 = require('../../common/services/logger.service');
const logger_service_mock_1 = require('../../common/services/logger.service.mock');
const abstract_server_1 = require('../servers/abstract.server');
const entityRegistry_1 = require('../../common/registry/entityRegistry');
const bootstrap_1 = require('./bootstrap');
const auth_service_1 = require('../services/auth.service');
const auth_service_mock_1 = require('../services/auth.service.mock');
let loggerInstance;
const providers = [
    {
        provide: logger_service_1.Logger,
        deps: [],
        useFactory: () => {
            if (!loggerInstance) {
                loggerInstance = new logger_service_mock_1.LoggerMock();
            }
            return loggerInstance;
        }
    },
    { provide: abstract_server_1.Server, useClass: abstract_server_mock_1.ServerMock },
    { provide: remoteCli_service_1.RemoteCli, useClass: remoteCli_service_mock_1.RemoteCliMock },
    { provide: auth_service_1.AuthService, useClass: auth_service_mock_1.AuthServiceMock },
];
describe('Bootstrap', () => {
    beforeEach(() => {
        entityRegistry_1.EntityRegistry.clearAll();
    });
    it('resolves server, logger and injector from providers', (done) => {
        const result = bootstrap_1.bootstrap(undefined, providers)();
        return result.then((res) => {
            expect(res.logger instanceof logger_service_1.Logger)
                .toBe(true, "bootstrap response logger is not instanceof Logger");
            expect(res.server instanceof abstract_server_1.Server)
                .toBe(true, "bootstrap response server is not instanceof Server");
            // Cannot check instanceof Injector for some reason, instead test that the injector works
            expect(res.injector.get(abstract_server_1.Server))
                .toEqual(res.server);
            done();
        });
    });
    it('calls an afterBootstrap function when provided', (done) => {
        const afterBootstrapFn = jasmine.createSpy('afterBootstrapFn');
        const result = bootstrap_1.bootstrap(undefined, providers, afterBootstrapFn)();
        return result.then((res) => {
            expect(afterBootstrapFn)
                .toHaveBeenCalledWith(res);
            done();
        });
    });
    it('can buffer logs created before the logger is instantiated, then invoke the logger with the buffer', (done) => {
        bootstrap_1.deferredLog('debug', 'this is a debug message');
        const loggerSpy = spyOn(loggerInstance, 'persistLog')
            .and
            .callThrough();
        spyOn(loggerInstance, 'source')
            .and
            .callFake(() => loggerInstance);
        const result = bootstrap_1.bootstrap(undefined, providers)();
        return result.then((res) => {
            expect(loggerSpy)
                .toHaveBeenCalledWith('debug', ['this is a debug message']);
            done();
        });
    });
    it('aborts startup when an error is encountered after bootstrappers ran, and logs error', (done) => {
        //as there is a fallback to output to log when a fatal bootstrap happens even when mocked,
        //here we spy on the console to suppress the log output
        const consoleErrorSpy = spyOn(console, 'error');
        spyOn(console, 'log');
        const error = new Error('Something is not right!');
        const afterBootstrapFn = () => {
            bootstrap_1.deferredLog('debug', 'this is a debug message');
            throw error;
        };
        const processExitSpy = spyOn(process, 'exit');
        const loggerSpy = spyOn(loggerInstance, 'persistLog')
            .and
            .callThrough();
        spyOn(loggerInstance, 'source')
            .and
            .callFake(() => loggerInstance);
        const result = bootstrap_1.bootstrap(undefined, providers, afterBootstrapFn)();
        return result.then((res) => {
            expect(processExitSpy)
                .toHaveBeenCalledWith(1);
            expect(loggerSpy)
                .toHaveBeenCalledWith('critical', ['Error', 'Something is not right!']);
            expect(loggerSpy)
                .toHaveBeenCalledWith('debug', ['this is a debug message']);
            expect(consoleErrorSpy)
                .toHaveBeenCalledWith(error);
            done();
        });
    });
    it('aborts startup when an error is before providers resolve, and logs with fallback console', (done) => {
        bootstrap_1.deferredLog('debug', 'this is a debug message');
        const processExitSpy = spyOn(process, 'exit');
        const consoleErrorSpy = spyOn(console, 'error');
        const consoleLogSpy = spyOn(console, 'log');
        const providersWithError = providers.concat([new Error]);
        const result = bootstrap_1.bootstrap(undefined, providersWithError)();
        return result.then((res) => {
            expect(processExitSpy)
                .toHaveBeenCalledWith(1);
            expect(consoleErrorSpy)
                .toHaveBeenCalled();
            expect(consoleLogSpy)
                .toHaveBeenCalledWith('debug', 'this is a debug message');
            done();
        });
    });
    it('logs the classes that were loaded', (done) => {
        const loggerSpy = spyOn(loggerInstance, 'persistLog')
            .and
            .callThrough();
        spyOn(loggerInstance, 'source')
            .and
            .callFake(() => loggerInstance);
        const classMap = [{ FooController: null }];
        const result = bootstrap_1.bootstrap(classMap, providers)();
        return result.then((res) => {
            expect(loggerSpy)
                .toHaveBeenCalledWith('debug', ['Classes loaded from app', ['FooController']]);
            done();
        });
    });
});
//# sourceMappingURL=bootstrap.spec.js.map