"use strict";
const logger_service_1 = require('../../common/services/logger.service');
const entityRegistry_1 = require('../../common/registry/entityRegistry');
/**
 * Provides abstract class for all bootstrappers to extend with, common interface for the
 * bootstrap function to have a single pattern to invoke the bootstrappers
 */
class EntityBootstrapper {
    /**
     * Kick off the bootstrapping function. The logger instance is assigned here as the injector is
     * not available at constructor time.
     * @returns {void|Promise<void>}
     */
    invokeBootstrap() {
        this.logger = this.injector.get(logger_service_1.Logger)
            .source(this.constructor.name);
        return this.bootstrap();
    }
    /**
     * Assign the current Injector to the class
     * @param injector
     * @returns {EntityBootstrapper}
     */
    setInjector(injector) {
        this.injector = injector;
        return this;
    }
    /**
     * Resolve & retrieve an instance of the entity from the injector. Note that depending on the
     * provider definition, this could be a different class to the passed token
     * @param token
     * @returns {T}
     */
    getInstance(token) {
        const instance = this.injector.get(token);
        this.logger.info(`Resolved ${instance.constructor.name}`);
        return instance;
    }
    /**
     * Retrieve all entities of the given type, converted from Map to Array
     * @param type
     * @returns {any[]}
     */
    getFromRegistry(type) {
        return [...entityRegistry_1.EntityRegistry.root.getAllOfType(type).values()];
    }
    /**
     * Retrieve the entities from the registry.
     *
     * @param type
     * @returns {RegistryEntityStatic[]}
     */
    getEntitiesFromRegistry(type) {
        // Note the apparent redundancy in assigning `this.entities` is intentional as the `entities`
        // property is used in the `getInjectableEntities` method of some implementations
        this.entities = this.getFromRegistry(type);
        return this.entities;
    }
}
exports.EntityBootstrapper = EntityBootstrapper;
//# sourceMappingURL=entity.bootstrapper.js.map