"use strict";
/**
 * @module server
 */
/** End Typedoc Module Declaration */
const entity_bootstrapper_1 = require('./entity.bootstrapper');
const typeorm_1 = require('typeorm');
const typeorm_2 = require('typeorm');
/**
 * Provides bootstrapping of the @[[Model]] entities
 */
class ModelBootstrapper extends entity_bootstrapper_1.EntityBootstrapper {
    /**
     * Models are not injectable, so this method simply returns an empty array
     * @returns {Array}
     */
    getInjectableEntities() {
        return [];
    }
    /**
     * Bootstraps the models. Each model has it's metadata mapped to the decorators from TypeORM
     * which are invoked to register their metadata with the internal TypeORM. This registry is later
     * used by the ModelStores which manipulate the TypeORM repository
     */
    bootstrap() {
        this.getFromRegistry('model')
            .forEach((model) => {
            const meta = model.getMetadata();
            this.logger.info(`initializing ${model.name}`, meta);
            typeorm_1.Table(meta.storageKey, meta.tableOptions)(model);
            for (const [property, definition] of meta.storedProperties) {
                if (property === meta.identifierKey) {
                    typeorm_2.PrimaryColumn(definition.columnOptions)(model.prototype, property);
                }
                else {
                    typeorm_2.Column(definition.columnOptions)(model.prototype, property);
                }
            }
            if (meta.timestamps) {
                if (meta.timestamps.updated) {
                    typeorm_2.UpdateDateColumn()(model.prototype, meta.timestamps.updated);
                }
                if (meta.timestamps.created) {
                    typeorm_2.CreateDateColumn()(model.prototype, meta.timestamps.created);
                }
            }
        });
    }
}
exports.ModelBootstrapper = ModelBootstrapper;
//# sourceMappingURL=models.bootstrapper.js.map