"use strict";
const exceptions_1 = require('../../common/exceptions/exceptions');
/**
 * Request class that is passed into all middleware and controller methods to extract data from the
 * client request.
 */
class Request {
    constructor(raw = undefined, paramsMap = new Map(), headersMap = new Map()) {
        this.raw = raw;
        this.paramsMap = paramsMap;
        this.headersMap = headersMap;
    }
    /**
     * Get the raw IncomingMessage
     * @returns {IncomingMessage}
     */
    getRaw() {
        return this.raw;
    }
    /**
     * Get all headers as a Map
     * @returns {Map<string, string>}
     */
    headers() {
        return this.headersMap;
    }
    /**
     * Get all route paramaters as a Map
     * @returns {Map<string, string>}
     */
    params() {
        return this.paramsMap;
    }
    /**
     * Helper function for converting Dictionary to Map
     * @param dictionary
     * @returns {Map<any, any>}
     */
    static extractMapFromDictionary(dictionary) {
        let map = new Map();
        for (let key in dictionary) {
            map.set(key, dictionary[key]);
        }
        return map;
    }
    /**
     * Extract the passed body object from the raw request
     * @returns {Promise<T>}
     */
    getPayload() {
        return new Promise((resolve, reject) => {
            this.raw.setEncoding('utf8');
            let data = '';
            this.raw.on(Request.EVENT_DATA, (d) => {
                data += d;
                if (data.length > 1e6) {
                    data = "";
                    let e = new exceptions_1.PayloadTooLargeException();
                    this.raw.socket.destroy();
                    reject(e);
                }
            });
            this.raw.on(Request.EVENT_END, () => {
                try {
                    resolve(JSON.parse(data));
                }
                catch (e) {
                    reject(new exceptions_1.UnprocessableEntityException());
                }
            });
            this.raw.on(Request.EVENT_ERROR, reject);
        });
    }
}
/** @event Emitted when request body data is read */
Request.EVENT_DATA = 'data';
/** @event Emitted when request body read is completed */
Request.EVENT_END = 'end';
/** @event Emitted when response body read is completed */
Request.EVENT_ERROR = 'end';
exports.Request = Request;
//# sourceMappingURL=request.js.map