"use strict";
const request_1 = require('../controllers/request');
const events_1 = require('events');
const exceptions_1 = require('../../common/exceptions/exceptions');
describe('Request', () => {
    it('Returns promise of the payload data', (done) => {
        const payload = { message: 'hello there' };
        let emitter = new events_1.EventEmitter();
        emitter.setEncoding = () => null;
        let request = new request_1.Request(emitter);
        request.getPayload()
            .then((result) => {
            expect(result)
                .toEqual(payload);
            done();
        });
        process.nextTick(() => {
            emitter.emit(request_1.Request.EVENT_DATA, JSON.stringify(payload));
            emitter.emit(request_1.Request.EVENT_END);
        });
    });
    it('Throws exception when payload is not json', (done) => {
        let emitter = new events_1.EventEmitter();
        emitter.setEncoding = () => null;
        let request = new request_1.Request(emitter);
        request.getPayload()
            .catch((error) => {
            expect(error instanceof exceptions_1.UnprocessableEntityException)
                .toBe(true);
            done();
        });
        process.nextTick(() => {
            emitter.emit(request_1.Request.EVENT_DATA, 'definitely_not_json');
            emitter.emit(request_1.Request.EVENT_END);
        });
    });
    it('Throws exception when payload is too large', (done) => {
        let emitter = new events_1.EventEmitter();
        emitter.setEncoding = () => null;
        const destroySpy = jasmine.createSpy('socket_destroy');
        emitter.socket = {
            destroy: destroySpy
        };
        let request = new request_1.Request(emitter);
        request.getPayload()
            .catch((error) => {
            expect(error instanceof exceptions_1.PayloadTooLargeException)
                .toBe(true);
            done();
            expect(destroySpy)
                .toHaveBeenCalled();
        });
        process.nextTick(() => {
            emitter.emit(request_1.Request.EVENT_DATA, 'a'.repeat(1e6 + 1)); //this may be a fairly intensive test, not sure
            // about the impact
        });
    });
    it('has utility function for converting flat dictionary to map', () => {
        const dict = {
            a: 1,
            b: 2
        };
        const map = request_1.Request.extractMapFromDictionary(dict);
        expect(map.get('a'))
            .toEqual(1);
        expect(map.get('b'))
            .toEqual(2);
    });
    it('can get the raw event emitter', () => {
        let emitter = new events_1.EventEmitter();
        let request = new request_1.Request(emitter);
        expect(request.getRaw())
            .toEqual(emitter);
    });
    it('can get the params map', () => {
        let request = new request_1.Request(null, new Map([['id', '1']]));
        expect(request.params()
            .get('id'))
            .toEqual('1');
    });
    it('can get the headers map', () => {
        let request = new request_1.Request(null, null, new Map([['Authorization', 'Basic: something']]));
        expect(request.headers()
            .get('Authorization'))
            .toEqual('Basic: something');
    });
});
//# sourceMappingURL=request.spec.js.map