"use strict";
/**
 * @module server
 */
/** End Typedoc Module Declaration */
const events_1 = require('events');
/**
 * Response class that is passed into all middleware and controller methods to be manipulated
 * and eventually dispatched to the client.
 *
 * Response is also an `EventEmitter` so you can attach listeners to the response:
 * * [[Response.EVENT_DATA]]
 * * [[Response.EVENT_END]]
 *
 */
class Response extends events_1.EventEmitter {
    constructor() {
        super(...arguments);
        /** The HTTP status code that is sent */
        this.statusCode = 200;
        /** Map of headers to send back */
        this.headers = new Map();
    }
    /**
     * Set the data to be sent back in the body.
     * This also emits a `data` event containing the response body and immediately sends `end` event
     * @param data
     * @returns {Response}
     */
    data(data) {
        this.body = data;
        this.emit(Response.EVENT_DATA, data);
        this.emit(Response.EVENT_END);
        return this;
    }
    /**
     * Set the status code to be sent
     * @param code
     * @returns {Response}
     */
    status(code) {
        this.statusCode = code;
        return this;
    }
    /**
     * Add or overwrite a header
     * @param name
     * @param value
     * @returns {Response}
     */
    header(name, value) {
        this.headers.set(name, value);
        return this;
    }
    /**
     * Utility method to send the correct status code for created entity
     * @param data
     * @returns {Response}
     */
    created(data) {
        return this
            .status(201)
            .data(data);
    }
}
/** @event Emitted when response body data is set */
Response.EVENT_DATA = 'data';
/** @event Emitted when response body setting is completed */
Response.EVENT_END = 'end';
exports.Response = Response;
//# sourceMappingURL=response.js.map