"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
const entityRegistry_1 = require('../../common/registry/entityRegistry');
const decorators_1 = require('../../common/registry/decorators');
const model_1 = require('../../common/models/model');
const decorators_2 = require('./decorators');
describe('Entity registry', () => {
    class Foo {
    }
    class Bar {
    }
    class Baz {
    }
    class Quux {
    }
    let testRegistry;
    beforeEach(() => {
        testRegistry = new entityRegistry_1.EntityRegistry;
    });
    it('registers an entity to the registry', () => {
        testRegistry.register('controller', Foo);
        let retrieved = testRegistry.registry.get('controller')
            .get('Foo');
        expect(retrieved)
            .toEqual(Foo);
        expect(retrieved)
            .not
            .toEqual(Bar);
    });
    it('gets all of one type', () => {
        testRegistry
            .register('controller', Foo)
            .register('controller', Bar)
            .register('model', Bar);
        const retrieved = testRegistry.getAllOfType('controller');
        expect(retrieved.size)
            .toEqual(2);
        expect([...retrieved.values()])
            .toEqual([Foo, Bar]);
    });
    it('returns empty map when there are none of type', () => {
        const retrieved = testRegistry.getAllOfType('controller');
        expect(retrieved instanceof Map)
            .toBe(true);
        expect(retrieved.size)
            .toBe(0);
    });
    it('returns empty map when items have been registered and removed', () => {
        testRegistry
            .register('controller', Foo);
        testRegistry.removeByType('controller', Foo.name);
        const retrieved = testRegistry.getAllOfType('controller');
        expect(retrieved instanceof Map)
            .toBe(true);
        expect(retrieved.size)
            .toBe(0);
    });
    it('clears all of a single type', () => {
        testRegistry
            .register('controller', Foo)
            .register('controller', Bar)
            .register('model', Bar);
        testRegistry.clearType('controller');
        const retrieved = testRegistry.getAllOfType('controller');
        expect(retrieved instanceof Map)
            .toBe(true);
        expect(retrieved.size)
            .toBe(0);
        expect(testRegistry.findByType('model', Bar.name))
            .toBe(Bar);
    });
    it('clears the registry', () => {
        testRegistry
            .register('controller', Foo)
            .register('controller', Bar)
            .register('model', Bar);
        testRegistry.clearAll();
        expect(testRegistry.registry instanceof Map)
            .toBe(true);
        expect(testRegistry.registry.size)
            .toBe(0);
    });
    it('checks if an entity exists', () => {
        testRegistry
            .register('controller', Foo)
            .register('controller', Bar)
            .register('model', Bar);
        expect(testRegistry.hasEntity('controller', Foo.name))
            .toBe(true);
        expect(testRegistry.hasEntity('model', Bar.name))
            .toBe(true);
        expect(testRegistry.hasEntity('model', Foo.name))
            .toBe(false);
    });
    it('finds all entities by name', () => {
        testRegistry
            .register('controller', Foo)
            .register('controller', Bar)
            .register('model', Bar);
        const all = testRegistry.findAllWithName(Bar.name);
        expect(all instanceof Map)
            .toBe(true);
        expect(all.size)
            .toBe(2);
        expect([...all.keys()])
            .toEqual(['controller', 'model']);
        expect([...all.values()])
            .toEqual([Bar, Bar]);
    });
    it('returns null when no entities found', () => {
        testRegistry
            .register('controller', Foo)
            .register('controller', Bar)
            .register('model', Bar);
        const all = testRegistry.findAllWithName('not_in_registry');
        expect(all)
            .toBe(null);
    });
    describe('decorators', () => {
        beforeAll(() => {
            let FooModel = class FooModel {
            };
            FooModel = __decorate([
                decorators_1.Model(), 
                __metadata('design:paramtypes', [])
            ], FooModel);
            let FooController = class FooController {
            };
            FooController = __decorate([
                decorators_2.Controller(), 
                __metadata('design:paramtypes', [])
            ], FooController);
            let FooSeeder = class FooSeeder {
            };
            FooSeeder = __decorate([
                decorators_2.Seeder(), 
                __metadata('design:paramtypes', [])
            ], FooSeeder);
            let FooMigration = class FooMigration {
            };
            FooMigration = __decorate([
                decorators_2.Migration(), 
                __metadata('design:paramtypes', [])
            ], FooMigration);
            let FooStore = class FooStore {
            };
            FooStore = __decorate([
                decorators_1.Store(), 
                __metadata('design:paramtypes', [])
            ], FooStore);
            let FooService = class FooService {
            };
            FooService = __decorate([
                decorators_1.Service(), 
                __metadata('design:paramtypes', [])
            ], FooService);
        });
        afterAll(() => {
            entityRegistry_1.EntityRegistry.root.clearAll();
        });
        [
            'Model',
            'Controller',
            'Seeder',
            'Migration',
            'Store',
            'Service',
        ].forEach((decorator) => {
            let type = decorator.toLowerCase();
            it(`decorates class with @${decorator} to register class as type '${type}'`, () => {
                let className = 'Foo' + decorator;
                let foundClass = entityRegistry_1.EntityRegistry.root.getAllOfType(type)
                    .get(className);
                expect(foundClass instanceof Function)
                    .toBe(true, typeof foundClass);
                expect(foundClass.name)
                    .toEqual(className);
            });
        });
        it('registers any passed metadata to the registry', () => {
            let BarModel = class BarModel extends model_1.AbstractModel {
            };
            BarModel = __decorate([
                decorators_1.Model({
                    storageKey: 'example'
                }), 
                __metadata('design:paramtypes', [])
            ], BarModel);
            let foundClass = entityRegistry_1.EntityRegistry.root.findByType('model', BarModel.name);
            expect(foundClass.getMetadata()).toEqual({ storageKey: 'example' });
        });
    });
});
//# sourceMappingURL=entityRegistry.spec.js.map