"use strict";
const abstract_server_1 = require('./abstract.server');
const remoteCli_service_1 = require('../services/remoteCli.service');
const logger_service_1 = require('../../common/services/logger.service');
const testing_1 = require('@angular/core/testing');
const logger_service_mock_1 = require('../../common/services/logger.service.mock');
const proxyquire = require('proxyquire');
const response_1 = require('../controllers/response');
const request_1 = require('../controllers/request');
const remoteCli_service_mock_1 = require('../services/remoteCli.service.mock');
const auth_service_mock_1 = require('../services/auth.service.mock');
const auth_service_1 = require('../services/auth.service');
describe('Express Server', () => {
    const expressConstructorSpy = jasmine.createSpy('expressConstructor');
    const expressSpy = jasmine.createSpyObj('express', ['get', 'use']);
    expressConstructorSpy.and.returnValue(expressSpy);
    const httpCreateServerSpy = jasmine.createSpy('httpCreateServer');
    const httpServerSpy = jasmine.createSpyObj('httpServer', ['listen']);
    httpServerSpy.listen = jasmine.createSpy('httpServerListener');
    httpCreateServerSpy.and.returnValue(httpServerSpy);
    const mockedModule = proxyquire('./express.server', {
        express: expressConstructorSpy,
        http: {
            createServer: httpCreateServerSpy
        }
    });
    const providers = [
        {
            provide: abstract_server_1.Server,
            deps: [logger_service_1.Logger, remoteCli_service_1.RemoteCli],
            useFactory: (logger, cli) => {
                return new mockedModule.ExpressServer(logger, cli);
            }
        },
        { provide: logger_service_1.Logger, useClass: logger_service_mock_1.LoggerMock },
        { provide: remoteCli_service_1.RemoteCli, useClass: remoteCli_service_mock_1.RemoteCliMock },
        { provide: auth_service_1.AuthService, useClass: auth_service_mock_1.AuthServiceMock },
    ];
    beforeEach(() => {
        testing_1.TestBed.configureTestingModule({ providers });
    });
    afterEach(() => {
        delete process.env.WEB_ROOT;
    });
    it('initialized http server with new express instance', testing_1.inject([abstract_server_1.Server], (server) => {
        expect(expressConstructorSpy)
            .toHaveBeenCalled();
        expect(httpCreateServerSpy)
            .toHaveBeenCalled();
        expect(server.getEngine())
            .toEqual(expressSpy);
    }));
    it('kicks off an http server when started', testing_1.async(testing_1.inject([abstract_server_1.Server], (server) => {
        const startPromise = server.start()
            .then((res) => {
            expect(res)
                .toEqual(server);
        });
        expect(httpServerSpy.listen)
            .toHaveBeenCalledWith(server.port, server.host, jasmine.any(Function));
        const startedCallback = httpServerSpy.listen.calls.mostRecent().args[2];
        startedCallback(); //resolve the promise
        return startPromise;
    })));
    it('registers routes with the engine, and dispatches calls to the express engine', testing_1.async(testing_1.inject([abstract_server_1.Server], (server) => {
        const callStackHandlerSpy = jasmine.createSpy('callStackHandler');
        const responseFixture = new response_1.Response().data('Hello World')
            .header('Content-Length', '11');
        callStackHandlerSpy.and.returnValue(Promise.resolve(responseFixture));
        const routeConfig = {
            path: '/test',
            methodName: 'test',
            method: 'GET',
            callStack: [],
            callStackHandler: callStackHandlerSpy
        };
        server.register(routeConfig);
        expect(expressSpy.get)
            .toHaveBeenCalledWith(routeConfig.path, jasmine.any(Function));
        const registrationCallback = expressSpy.get.calls.mostRecent().args[1];
        const reqSpy = jasmine.createSpy('req')
            .and
            .returnValue({
            params: {
                id: 1,
            },
            header: {
                Authorization: 'Bearer: Banana'
            }
        });
        const resSpy = jasmine.createSpyObj('res', ['status', 'send', 'header']);
        resSpy.status = jasmine.createSpy('res_status');
        resSpy.send = jasmine.createSpy('res_send');
        resSpy.header = jasmine.createSpy('res_send');
        return registrationCallback(reqSpy, resSpy)
            .then(() => {
            expect(callStackHandlerSpy)
                .toHaveBeenCalledWith(jasmine.any(request_1.Request), jasmine.any(response_1.Response));
            expect(resSpy.status)
                .toHaveBeenCalledWith(200);
            expect(resSpy.send)
                .toHaveBeenCalledWith(responseFixture.body);
            expect(resSpy.header)
                .toHaveBeenCalledWith('Content-Length', '11');
        });
    })));
    it('registers routes with the engine, and dispatches errors to the express engine', testing_1.async(testing_1.inject([abstract_server_1.Server], (server) => {
        const callStackHandlerSpy = jasmine.createSpy('callStackHandler');
        const responseFixture = new Error('Internal error');
        callStackHandlerSpy.and.returnValue(Promise.reject(responseFixture));
        const routeConfig = {
            path: '/test',
            methodName: 'test',
            method: 'GET',
            callStack: [],
            callStackHandler: callStackHandlerSpy
        };
        server.register(routeConfig);
        expect(expressSpy.get)
            .toHaveBeenCalledWith(routeConfig.path, jasmine.any(Function));
        const registrationCallback = expressSpy.get.calls.mostRecent().args[1];
        const reqSpy = jasmine.createSpy('req')
            .and
            .returnValue({
            params: null,
            header: null
        });
        const resSpy = jasmine.createSpyObj('res', ['status', 'send', 'header']);
        resSpy.status = jasmine.createSpy('res_status');
        resSpy.send = jasmine.createSpy('res_send');
        resSpy.header = jasmine.createSpy('res_send');
        resSpy.statusCode = 200;
        return registrationCallback(reqSpy, resSpy)
            .then(() => {
            expect(callStackHandlerSpy)
                .toHaveBeenCalledWith(jasmine.any(request_1.Request), jasmine.any(response_1.Response));
            expect(resSpy.status)
                .toHaveBeenCalledWith(500);
            expect(resSpy.send)
                .toHaveBeenCalledWith(responseFixture);
        });
    })));
    it('registers static file loader when started', testing_1.async(testing_1.inject([abstract_server_1.Server], (server) => {
        process.env.WEB_ROOT = '/tmp/example';
        expressConstructorSpy['static'] = jasmine.createSpy('express_static');
        const startPromise = server.start()
            .then((res) => {
            expect(res)
                .toEqual(server);
            expect(expressConstructorSpy['static'])
                .toHaveBeenCalledWith('/tmp/example', { index: ['index.html'] });
        });
        const startedCallback = httpServerSpy.listen.calls.mostRecent().args[2];
        startedCallback(); //resolve the promise
        return startPromise;
    })));
});
//# sourceMappingURL=express.server.spec.js.map