"use strict";
const abstract_server_1 = require('./abstract.server');
const remoteCli_service_1 = require('../services/remoteCli.service');
const logger_service_1 = require('../../common/services/logger.service');
const testing_1 = require('@angular/core/testing');
const logger_service_mock_1 = require('../../common/services/logger.service.mock');
const remoteCli_service_mock_1 = require('../services/remoteCli.service.mock');
const proxyquire = require('proxyquire');
const response_1 = require('../controllers/response');
const request_1 = require('../controllers/request');
const auth_service_mock_1 = require('../services/auth.service.mock');
const auth_service_1 = require('../services/auth.service');
describe('Hapi Server', () => {
    const hapiConstructorSpy = jasmine.createSpy('hapiConstructor');
    const hapiSpy = jasmine.createSpyObj('hapi', ['start', 'connection', 'route']);
    hapiConstructorSpy.and.returnValue(hapiSpy);
    const mockedModule = proxyquire('./hapi.server', {
        hapi: {
            Server: hapiConstructorSpy
        },
    });
    const providers = [
        {
            provide: abstract_server_1.Server, deps: [logger_service_1.Logger, remoteCli_service_1.RemoteCli], useFactory: (logger, cli) => {
                return new mockedModule.HapiServer(logger, cli);
            }
        },
        { provide: logger_service_1.Logger, useClass: logger_service_mock_1.LoggerMock },
        { provide: remoteCli_service_1.RemoteCli, useClass: remoteCli_service_mock_1.RemoteCliMock },
        { provide: auth_service_1.AuthService, useClass: auth_service_mock_1.AuthServiceMock },
    ];
    beforeEach(() => {
        testing_1.TestBed.configureTestingModule({ providers });
    });
    afterEach(() => {
        delete process.env.WEB_ROOT;
    });
    it('initialized http server with new hapi instance', testing_1.inject([abstract_server_1.Server], (server) => {
        expect(hapiConstructorSpy)
            .toHaveBeenCalled();
        expect(hapiSpy.connection)
            .toHaveBeenCalledWith({ port: server.port, host: server.host });
        expect(server.getEngine())
            .toEqual(hapiSpy);
    }));
    it('throws error when webroot is defined as static file listing is not yet implemented', testing_1.inject([abstract_server_1.Server], (server) => {
        process.env.WEB_ROOT = '/tmp/example';
        const fixture = () => server.start();
        expect(fixture).toThrowError('Static file listing is not implemented for hapi');
    }));
    it('kicks off an http server when started', testing_1.async(testing_1.inject([abstract_server_1.Server], (server) => {
        const startPromise = server.start()
            .then((res) => {
            expect(res)
                .toEqual(server);
        });
        expect(hapiSpy.start)
            .toHaveBeenCalled();
        const startCallback = hapiSpy.start.calls.mostRecent().args[0];
        startCallback();
        return startPromise;
    })));
    it('rejects promise when server start fails', testing_1.async(testing_1.inject([abstract_server_1.Server], (server) => {
        const startPromise = server.start();
        expect(hapiSpy.start)
            .toHaveBeenCalled();
        const startCallback = hapiSpy.start.calls.mostRecent().args[0];
        const errorFixture = new Error('something went wrong');
        startCallback(errorFixture);
        return startPromise.catch((e) => {
            expect(e)
                .toEqual(errorFixture);
        });
    })));
    it('throws error when invalid route is set', testing_1.async(testing_1.inject([abstract_server_1.Server], (server) => {
        const routeConfig = {
            path: '/test/optional?',
            methodName: 'test',
            method: 'GET',
            callStack: [],
            callStackHandler: null
        };
        const expectedError = () => server.register(routeConfig);
        expect(expectedError)
            .toThrowError('Hapi syntax for optional or multi-segment parameters is not supported');
        expect(hapiSpy.route)
            .not
            .toHaveBeenCalled();
    })));
    it('registers routes with the engine, and dispatches calls to the hapi engine', testing_1.async(testing_1.inject([abstract_server_1.Server], (server) => {
        const callStackHandlerSpy = jasmine.createSpy('callStackHandler');
        const responseFixture = new response_1.Response().data('Hello World')
            .header('Content-Length', '11');
        callStackHandlerSpy.and.returnValue(Promise.resolve(responseFixture));
        const routeConfig = {
            path: '/test',
            methodName: 'test',
            method: 'GET',
            callStack: [],
            callStackHandler: callStackHandlerSpy
        };
        server.register(routeConfig);
        expect(hapiSpy.route)
            .toHaveBeenCalledWith({
            path: routeConfig.path,
            method: 'GET',
            handler: jasmine.any(Function)
        });
        const registrationCallback = hapiSpy.route.calls.mostRecent().args[0].handler;
        const reqFixture = {
            params: {
                id: 1,
            },
            headers: {
                Authorization: 'Bearer: Banana'
            },
            raw: {
                req: null
            }
        };
        const resSpy = jasmine.createSpyObj('res', ['code', 'header']);
        const replySpy = jasmine.createSpy('reply')
            .and
            .returnValue(resSpy);
        return registrationCallback(reqFixture, replySpy)
            .then((response) => {
            expect(callStackHandlerSpy)
                .toHaveBeenCalledWith(jasmine.any(request_1.Request), jasmine.any(response_1.Response));
            expect(resSpy.code)
                .toHaveBeenCalledWith(200);
            expect(resSpy.header)
                .toHaveBeenCalledWith('Content-Length', '11');
            expect(replySpy)
                .toHaveBeenCalledWith(responseFixture.body);
            expect(response)
                .toEqual(resSpy);
        });
    })));
    it('registers routes with the engine, and dispatches errors to the hapi engine', testing_1.async(testing_1.inject([abstract_server_1.Server], (server) => {
        const callStackHandlerSpy = jasmine.createSpy('callStackHandler');
        const responseFixture = new Error('Internal error');
        callStackHandlerSpy.and.returnValue(Promise.reject(responseFixture));
        const routeConfig = {
            path: '/test',
            methodName: 'test',
            method: 'GET',
            callStack: [],
            callStackHandler: callStackHandlerSpy
        };
        server.register(routeConfig);
        expect(hapiSpy.route)
            .toHaveBeenCalledWith({
            path: routeConfig.path,
            method: 'GET',
            handler: jasmine.any(Function)
        });
        const registrationCallback = hapiSpy.route.calls.mostRecent().args[0].handler;
        const reqFixture = {
            params: null,
            headers: null,
            raw: {
                req: null
            }
        };
        const resSpy = jasmine.createSpyObj('res', ['code', 'header']);
        const replySpy = jasmine.createSpy('reply')
            .and
            .returnValue(resSpy);
        resSpy.statusCode = 200;
        return registrationCallback(reqFixture, replySpy)
            .then((response) => {
            expect(callStackHandlerSpy)
                .toHaveBeenCalledWith(jasmine.any(request_1.Request), jasmine.any(response_1.Response));
            expect(resSpy.code)
                .toHaveBeenCalledWith(500);
            expect(response)
                .toEqual(resSpy);
        });
    })));
});
//# sourceMappingURL=hapi.server.spec.js.map