"use strict";
const proxyquire = require('proxyquire');
const logger_service_1 = require('../../common/services/logger.service');
const testing_1 = require('@angular/core/testing');
const logger_service_mock_1 = require('../../common/services/logger.service.mock');
const auth_service_mock_1 = require('./auth.service.mock');
const auth_service_1 = require('./auth.service');
const jwtSpy = jasmine.createSpyObj('jwt', ['verify', 'decode']);
const payload = {
    username: 'bob',
};
jwtSpy.verify.and.returnValue(Promise.resolve(payload));
jwtSpy.decode.and.returnValue(Promise.resolve(payload));
describe('Auth Service (JWT)', () => {
    const fileReadSpy = jasmine.createSpy('readFileSync');
    const pem = `
--- BEGIN FAKE KEY ---
--- END FAKE KEY ---
`;
    fileReadSpy.and.returnValue(pem);
    const mockedModule = proxyquire('./auth.service', {
        jsonwebtoken: jwtSpy,
        fs: {
            readFileSync: fileReadSpy
        }
    });
    const providers = [
        {
            provide: auth_service_1.AuthService,
            deps: [logger_service_1.Logger],
            useFactory: (logger) => {
                return new mockedModule.AuthService(logger);
            }
        },
        { provide: logger_service_1.Logger, useClass: logger_service_mock_1.LoggerMock },
    ];
    beforeEach(() => {
        testing_1.TestBed.configureTestingModule({ providers });
    });
    afterEach(() => {
        jwtSpy.verify.calls.reset();
    });
    it('verifies credentials with jwt and public key', testing_1.async(testing_1.inject([auth_service_1.AuthService], (s) => {
        const jwt = 'pretend.this.is.a.jwt';
        const publicKeyPath = './path/to/key';
        const params = {
            iat: 12345679
        };
        const resPromise = s.verify(jwt, publicKeyPath, params);
        expect(fileReadSpy)
            .toHaveBeenCalledWith(publicKeyPath);
        expect(jwtSpy.verify)
            .toHaveBeenCalledWith(jwt, pem, params, jasmine.any(Function));
        const callArgCb = jwtSpy.verify.calls.mostRecent().args[3];
        callArgCb(null, payload);
        return resPromise.then((res) => {
            expect(res)
                .toEqual(payload);
        });
    })));
    it('rejects verification of credentials when jwt.verify fails', testing_1.async(testing_1.inject([auth_service_1.AuthService], (s) => {
        const jwt = 'pretend.this.is.an.invalid.jwt';
        const publicKeyPath = './path/to/key';
        const resPromise = s.verify(jwt, publicKeyPath);
        expect(fileReadSpy)
            .toHaveBeenCalledWith(publicKeyPath);
        expect(jwtSpy.verify)
            .toHaveBeenCalledWith(jwt, pem, {}, jasmine.any(Function));
        const callArgCb = jwtSpy.verify.calls.mostRecent().args[3];
        callArgCb(new Error('jwt error'), payload);
        return resPromise.catch((e) => {
            expect(e.message)
                .toEqual('jwt error');
        });
    })));
});
describe('Auth Service (JWT) Mock', () => {
    const mockedModule = proxyquire('./auth.service.mock', {
        jsonwebtoken: jwtSpy,
    });
    const providers = [
        {
            provide: auth_service_mock_1.AuthServiceMock,
            deps: [logger_service_1.Logger],
            useFactory: (logger) => {
                return new mockedModule.AuthServiceMock(logger);
            }
        },
        { provide: logger_service_1.Logger, useClass: logger_service_mock_1.LoggerMock },
    ];
    beforeEach(() => {
        testing_1.TestBed.configureTestingModule({ providers });
    });
    it('returns decoded jwt directly without checking', testing_1.async(testing_1.inject([auth_service_mock_1.AuthServiceMock], (s) => {
        const jwt = 'pretend.this.is.a.valid.jwt';
        const resPromise = s.verify(jwt);
        expect(jwtSpy.decode)
            .toHaveBeenCalledWith(jwt);
        return resPromise.then((res) => {
            expect(res)
                .toEqual(payload);
        });
    })));
});
//# sourceMappingURL=auth.service.spec.js.map