"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator.throw(value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
/**
 * @module server
 */
/** End Typedoc Module Declaration */
const core_1 = require('@angular/core');
const logger_service_1 = require('../../common/services/logger.service');
const typeorm_1 = require('typeorm');
const entityRegistry_1 = require('../../common/registry/entityRegistry');
const decorators_1 = require('../../common/registry/decorators');
const service_1 = require('../../common/services/service');
const SQL = require('sql-template-strings');
/**
 * Core database service for connecting to the SQL db
 */
let Database_1 = class Database extends service_1.AbstractService {
    constructor(loggerBase) {
        super();
        this.logger = loggerBase.source('database');
        this.initialized = this.initialize();
    }
    /**
     * Connects to the database and stores reference to the connection instance
     * @returns {Promise<Database>}
     */
    initialize() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield Database_1.connect(undefined, (level, message) => this.logger[level](message));
            }
            catch (e) {
                this.logger.critical(e);
                throw e;
            }
            return this;
        });
    }
    /**
     * Connect to the database, returning promised of connection instance.
     * Static method so calls can be made outside of dependency injection context for example
     * pre-bootstrap
     * @param name
     * @param logFunction
     * @returns {Promise<Connection>}
     */
    static connect(name = 'default', logFunction) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.connections.has(name)) {
                return this.connections.get(name);
            }
            let logger;
            if (logFunction) {
                logFunction('info', 'Connecting to database');
                //remap log levels to zeroth' keys
                logger = (level, message) => {
                    switch (level) {
                        case 'log':
                            level = 'info';
                            break;
                        case 'warn':
                            level = 'warning';
                            break;
                    }
                    logFunction(level, message);
                };
            }
            const options = {
                name,
                driver: {
                    type: process.env.DB_DRIVER,
                    host: process.env.DB_HOST,
                    port: process.env.DB_PORT,
                    username: process.env.DB_USERNAME,
                    password: process.env.DB_PASSWORD,
                    database: process.env.DB_DATABASE,
                    usePool: false,
                },
                // created on each application start
                logging: {
                    logQueries: true,
                    logger,
                },
                autoSchemaSync: false,
                entities: [
                    ...entityRegistry_1.EntityRegistry.root.getAllOfType('model')
                        .values()
                ],
            };
            const connection = yield typeorm_1.createConnection(options);
            this.connections.set(name, connection);
            // await connection.driver.createQueryRunner();
            return connection;
        });
    }
    static clearConnections() {
        this.connections.clear();
    }
    /**
     * Retrieve connection instance
     * @returns {Promise<Connection>}
     */
    getConnection(name = 'default') {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.initialize();
            return Database_1.connections.get(name);
        });
    }
    /**
     * Execute a raw query
     * @param sql
     * @returns {Promise<any>}
     */
    query(sql) {
        return __awaiter(this, void 0, void 0, function* () {
            return (yield this.getQueryRunner()).query(sql);
        });
    }
    /**
     * Get current driver from connection for direct database interaction
     * @returns {Promise<Driver>}
     */
    getDriver(connectionName = 'default') {
        return __awaiter(this, void 0, void 0, function* () {
            return (yield this.getConnection(connectionName)).driver;
        });
    }
    /**
     * Get current query runner from current driver for direct database interaction
     * @returns {Promise<QueryRunner>}
     */
    getQueryRunner(connectionName = 'default') {
        return __awaiter(this, void 0, void 0, function* () {
            return (yield this.getDriver(connectionName)).createQueryRunner();
        });
    }
    /**
     * ES6 template string tagger to create prepared statements
     *
     * Example:
     * ```typescript
     * return this.database.query(Database.prepare`INSERT
     * INTO    books
     * (name, author, isbn, category, recommended_age, pages, price)
     * VALUES  (${name}, ${author}, ${isbn}, ${category}, ${recommendedAge}, ${pages}, ${price})
     * `);
     * ```
     * This will generate a prepared statement with all the safety features that come with that
     * @see https://www.npmjs.com/package/sql-template-strings
     * @param args
     * @returns {any}
     */
    static prepare(...args) {
        return SQL(...args);
    }
};
let Database = Database_1;
Database.connections = new Map();
Database = Database_1 = __decorate([
    core_1.Injectable(),
    decorators_1.Service(), 
    __metadata('design:paramtypes', [logger_service_1.Logger])
], Database);
exports.Database = Database;
//# sourceMappingURL=database.service.js.map