"use strict";
const proxyquire = require('proxyquire');
const core_1 = require('@angular/core');
const remoteCli_service_1 = require('./remoteCli.service');
const logger_service_1 = require('../../common/services/logger.service');
const testing_1 = require('@angular/core/testing');
const logger_service_mock_1 = require('../../common/services/logger.service.mock');
const entityRegistry_1 = require('../../common/registry/entityRegistry');
const abstract_server_1 = require('../servers/abstract.server');
const abstract_server_mock_1 = require('../servers/abstract.server.mock');
const remoteCli_service_mock_1 = require('./remoteCli.service.mock');
const auth_service_mock_1 = require('./auth.service.mock');
const auth_service_1 = require('./auth.service');
const chalk = require('chalk');
describe('Remote CLI Commands', () => {
    const vantageSpy = jasmine.createSpyObj('vantage', [
        'delimiter', 'banner', 'command', 'description', 'action', 'listen', 'auth'
    ]);
    //support chaining
    vantageSpy.command.and.returnValue(vantageSpy);
    vantageSpy.description.and.returnValue(vantageSpy);
    vantageSpy.action.and.returnValue(vantageSpy);
    const vantageConstructorSpy = jasmine.createSpy('vantageConstructor')
        .and
        .returnValue(vantageSpy);
    const tableSpy = jasmine.createSpy('table');
    const mockedModule = proxyquire('./remoteCli.service', {
        ['vantage']: vantageConstructorSpy,
        table: tableSpy,
    });
    const providers = [
        {
            provide: remoteCli_service_1.RemoteCli,
            deps: [logger_service_1.Logger, core_1.Injector, auth_service_1.AuthService],
            useFactory: (logger, injector, authService) => {
                return new mockedModule.RemoteCli(logger, injector, authService).initialize();
            }
        },
        { provide: logger_service_1.Logger, useClass: logger_service_mock_1.LoggerMock },
        { provide: abstract_server_1.Server, useClass: abstract_server_mock_1.ServerMock },
        { provide: auth_service_1.AuthService, useClass: auth_service_mock_1.AuthServiceMock },
    ];
    beforeEach(() => {
        testing_1.TestBed.configureTestingModule({ providers });
    });
    beforeEach(() => {
        entityRegistry_1.EntityRegistry.clearAll();
        process.stdout.columns = 90;
    });
    it('initializes cli with vantage', testing_1.inject([remoteCli_service_1.RemoteCli], (cli) => {
        expect(vantageConstructorSpy)
            .toHaveBeenCalled();
        expect(vantageSpy.delimiter)
            .toHaveBeenCalledWith(chalk.magenta('zeroth-runtime~$'));
        //the banner is not called on init otherwise it would be output pre-initialization
        expect(vantageSpy.banner)
            .not
            .toHaveBeenCalled();
        expect(vantageSpy.auth)
            .toHaveBeenCalled();
    }));
    it('starts a vantage server', testing_1.inject([remoteCli_service_1.RemoteCli], (cli) => {
        cli.start(1234);
        expect(vantageSpy.listen)
            .toHaveBeenCalledWith(1234, jasmine.any(Function));
        const callbackLogFunction = vantageSpy.listen.calls.mostRecent().args[1];
        const loggerSpy = spyOn(cli.logger, 'persistLog')
            .and
            .callThrough();
        callbackLogFunction({ conn: { remoteAddress: '127.0.0.1' } });
        expect(loggerSpy)
            .toHaveBeenCalledWith('info', [`Accepted a connection from [127.0.0.1]`]);
    }));
    it('registers route table output action on startup', testing_1.inject([remoteCli_service_1.RemoteCli, abstract_server_1.Server], (cli, server) => {
        expect(vantageSpy.action)
            .toHaveBeenCalledWith(jasmine.any(Function));
        const mockRoute = {
            path: '/test',
            methodName: 'test',
            method: 'GET',
            callStack: [
                function beforeMiddleware() {
                },
                function test() {
                },
                function afterMiddleware() {
                },
            ],
            callStackHandler: null
        };
        spyOn(server, 'getRoutes')
            .and
            .returnValue([mockRoute]);
        const actionFunction = vantageSpy.action.calls.mostRecent().args[0];
        const actionLogSpy = jasmine.createSpy('action_logger');
        const boundAction = actionFunction.bind({
            log: actionLogSpy
        });
        const makeTableSpy = spyOn(cli, 'makeTable')
            .and
            .callThrough();
        boundAction(null, () => null);
        expect(makeTableSpy)
            .toHaveBeenCalled();
        const tableArg = makeTableSpy.calls.mostRecent().args[0];
        expect(tableArg[0].toString())
            .toContain('Method');
        expect(tableArg[1])
            .toEqual(['GET', '/test', ['beforeMiddleware', 'test', 'afterMiddleware']]);
        expect(actionLogSpy)
            .toHaveBeenCalled();
    }));
});
describe('Remote Command Mock', () => {
    const providers = [
        remoteCli_service_mock_1.RemoteCliMock,
        { provide: logger_service_1.Logger, useClass: logger_service_mock_1.LoggerMock },
        { provide: abstract_server_1.Server, useClass: abstract_server_mock_1.ServerMock },
        { provide: auth_service_1.AuthService, useClass: auth_service_mock_1.AuthServiceMock },
    ];
    beforeEach(() => {
        testing_1.TestBed.configureTestingModule({ providers });
    });
    it('mocks the interfaces of RemoteCli', testing_1.inject([remoteCli_service_mock_1.RemoteCliMock], (cli) => {
        const spy = spyOn(cli, 'registerCommands')
            .and
            .callThrough();
        const start = cli.initialize()
            .start(1234);
        expect(start)
            .toEqual(cli);
        expect(spy)
            .toHaveBeenCalled();
    }));
});
//# sourceMappingURL=remoteCli.service.spec.js.map